"""Metadata for autopypath package.

This file contains shared metadata for the project.
It is parsed by the build system and should only contain simple assignments
(str, int, float, bool, None).
"""

__author__ = 'Jerilyn Franz'
__copyright__ = '2026, Jerilyn Franz'
__version__ = '0.9.1'
__release__ = '0.9.1'
__project__ = 'autopypath'
__license__ = 'Apache-2.0'
__url__ = 'https://github.com/snowhare/python-autopypath'
